/// <reference types="node" />
import { Beet, BeetField, SupportedTypeDefinition } from './types';
import { CollectionsExports, CollectionsTypeMapKey } from './collections';
import { CompositesExports, CompositesTypeMapKey } from './composites';
import { NumbersExports, NumbersTypeMapKey } from './numbers';
export * from './collections';
export * from './composites';
export * from './numbers';
export * from './types';
export declare type BeetTypeMapKey = CollectionsTypeMapKey | CompositesTypeMapKey | NumbersTypeMapKey;
export declare type BeetExports = CollectionsExports | CompositesExports | NumbersExports;
export declare const supportedTypeMap: Record<BeetTypeMapKey, SupportedTypeDefinition & {
    beet: BeetExports;
}>;
export declare class BeetWriter {
    private buf;
    private _offset;
    constructor(byteSize: number);
    get buffer(): Buffer;
    get offset(): number;
    private maybeResize;
    write<T>(beet: Beet<T>, value: T): void;
    writeStruct<T>(instance: T, fields: BeetField<T>[]): void;
}
export declare class BeetReader {
    private readonly buffer;
    private _offset;
    constructor(buffer: Buffer, _offset?: number);
    get offset(): number;
    read<T>(beet: Beet<T>): T;
    readStruct<T>(fields: BeetField<T>[]): T;
}
export declare type BeetOptions<Class> = {
    byteSize: (instance: Class) => number;
    postSerialize: (instance: Class, buf: Buffer, offset: number) => [Buffer, number];
};
export declare class BeetStruct<Class, Args = Partial<Class>> implements Beet<Class> {
    private readonly fields;
    private readonly construct;
    readonly description: string;
    readonly byteSize: number;
    constructor(fields: BeetField<Args>[], construct: (args: Args) => Class, description?: string);
    read(buf: Buffer, offset: number): Class;
    write(buf: Buffer, offset: number, value: Args): void;
    deserialize(buffer: Buffer, offset?: number): [Class, number];
    serialize(instance: Args, byteSize?: number): [Buffer, number];
    private getByteSize;
    static description: string;
}
/**
 * Convenience wrapper around {@link BeetStruct} which is used for plain JavasScript
 * objects, like are used for option args passed to functions.
 */
export declare class BeetArgsStruct<Args> extends BeetStruct<Args, Args> {
    constructor(fields: BeetField<Args>[], description?: string);
    static description: string;
}
