"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.compositesTypeMap = exports.dataEnum = exports.coption = void 0;
const assert_1 = require("assert");
const numbers_1 = require("./numbers");
const types_1 = require("./types");
const package_json_1 = require("../package.json");
const BEET_PACKAGE = package_json_1.name;
const SOME = Buffer.from(Uint8Array.from([1, 0, 0, 0])).slice(0, 4);
const NONE = Buffer.from(Uint8Array.from([0, 0, 0, 0])).slice(0, 4);
function coption(inner) {
    return {
        write: function (buf, offset, value) {
            if (value == null) {
                NONE.copy(buf, offset, 0, 4);
                // NOTE: here we leave the remaining part of the buffer unchanged
                // as it won't be consumed on read either.
                // Also it should be zero filled already.
            }
            else {
                SOME.copy(buf, offset, 0, 4);
                inner.write(buf, offset + 4, value);
            }
        },
        read: function (buf, offset) {
            if (buf.compare(NONE, 0, 4, offset, offset + 4) === 0) {
                return null;
            }
            (0, assert_1.strict)(buf.compare(SOME, 0, 4, offset, offset + 4) === 0, 'should be valid COption buffer');
            return inner.read(buf, offset + 4);
        },
        byteSize: 4 + inner.byteSize,
        description: `COption<${inner.description}>`,
    };
}
exports.coption = coption;
function dataEnum(inner) {
    return {
        write: function (buf, offset, value) {
            numbers_1.u8.write(buf, offset, value.kind);
            inner.write(buf, offset + 1, value.data);
        },
        read: function (buf, offset) {
            const kind = numbers_1.u8.read(buf, offset);
            const data = inner.read(buf, offset + 1);
            return { kind, data };
        },
        byteSize: 1 + inner.byteSize,
        description: `DataEnum<${inner.description}>`,
    };
}
exports.dataEnum = dataEnum;
// prettier-ignore
exports.compositesTypeMap = {
    option: { beet: 'coption', sourcePack: BEET_PACKAGE, ts: 'COption<Inner>', arg: types_1.BEET_TYPE_ARG_INNER, pack: BEET_PACKAGE },
    enum: { beet: 'dataEnum', sourcePack: BEET_PACKAGE, ts: 'DataEnum<Kind, Inner>', arg: types_1.BEET_TYPE_ARG_INNER, pack: BEET_PACKAGE }
};
//# sourceMappingURL=composites.js.map