"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.numbersTypeMap = exports.bool = exports.i32 = exports.i16 = exports.i8 = exports.u512 = exports.u256 = exports.u128 = exports.u64 = exports.u32 = exports.u16 = exports.u8 = void 0;
const bn_js_1 = __importDefault(require("bn.js"));
const package_json_1 = require("../package.json");
const BEET_PACKAGE = package_json_1.name;
// -----------------
// Unsigned
// -----------------
exports.u8 = {
    write: function (buf, offset, value) {
        buf.writeUInt8(value, offset);
    },
    read: function (buf, offset) {
        return buf.readUInt8(offset);
    },
    byteSize: 1,
    description: 'u8',
};
exports.u16 = {
    write: function (buf, offset, value) {
        buf.writeUInt16LE(value, offset);
    },
    read: function (buf, offset) {
        return buf.readUInt16LE(offset);
    },
    byteSize: 2,
    description: 'u16',
};
exports.u32 = {
    write: function (buf, offset, value) {
        buf.writeUInt32LE(value, offset);
    },
    read: function (buf, offset) {
        return buf.readUInt32LE(offset);
    },
    byteSize: 4,
    description: 'u32',
};
function unsignedLargeBeet(byteSize, description) {
    return {
        write: function (buf, offset, value) {
            const bn = bn_js_1.default.isBN(value) ? value : new bn_js_1.default(value);
            const bytesArray = bn.toArray('le', this.byteSize);
            const bytesArrayBuf = Buffer.from(bytesArray);
            bytesArrayBuf.copy(buf, offset, 0, this.byteSize);
        },
        read: function (buf, offset) {
            const slice = buf.slice(offset, offset + this.byteSize);
            return new bn_js_1.default(slice, 'le');
        },
        byteSize,
        description,
    };
}
exports.u64 = unsignedLargeBeet(8, 'u64');
exports.u128 = unsignedLargeBeet(16, 'u128');
exports.u256 = unsignedLargeBeet(32, 'u256');
exports.u512 = unsignedLargeBeet(64, 'u512');
// -----------------
// Signed
// -----------------
exports.i8 = {
    write: function (buf, offset, value) {
        buf.writeInt8(value, offset);
    },
    read: function (buf, offset) {
        return buf.readInt8(offset);
    },
    byteSize: 1,
    description: 'i8',
};
exports.i16 = {
    write: function (buf, offset, value) {
        buf.writeInt16LE(value, offset);
    },
    read: function (buf, offset) {
        return buf.readInt16LE(offset);
    },
    byteSize: 2,
    description: 'i16',
};
exports.i32 = {
    write: function (buf, offset, value) {
        buf.writeInt32LE(value, offset);
    },
    read: function (buf, offset) {
        return buf.readInt32LE(offset);
    },
    byteSize: 4,
    description: 'i32',
};
// -----------------
// Boolean
// -----------------
exports.bool = {
    write: function (buf, offset, value) {
        const n = value ? 1 : 0;
        exports.u8.write(buf, offset, n);
    },
    read: function (buf, offset) {
        return exports.u8.read(buf, offset) === 1;
    },
    byteSize: 1,
    description: 'bool',
};
// prettier-ignore
exports.numbersTypeMap = {
    // <= 32-bit numbers and boolean
    u8: { beet: 'u8', sourcePack: BEET_PACKAGE, ts: 'number' },
    u16: { beet: 'u16', sourcePack: BEET_PACKAGE, ts: 'number' },
    u32: { beet: 'u32', sourcePack: BEET_PACKAGE, ts: 'number' },
    i8: { beet: 'i8', sourcePack: BEET_PACKAGE, ts: 'number' },
    i16: { beet: 'i16', sourcePack: BEET_PACKAGE, ts: 'number' },
    i32: { beet: 'i32', sourcePack: BEET_PACKAGE, ts: 'number' },
    bool: { beet: 'bool', sourcePack: BEET_PACKAGE, ts: 'boolean' },
    // Big Number, they use, the 'bignum' type which is defined in this package
    u64: { beet: 'u64', sourcePack: BEET_PACKAGE, ts: 'bignum', pack: BEET_PACKAGE },
    u128: { beet: 'u128', sourcePack: BEET_PACKAGE, ts: 'bignum', pack: BEET_PACKAGE },
    u256: { beet: 'u256', sourcePack: BEET_PACKAGE, ts: 'bignum', pack: BEET_PACKAGE },
    u512: { beet: 'u512', sourcePack: BEET_PACKAGE, ts: 'bignum', pack: BEET_PACKAGE },
};
//# sourceMappingURL=numbers.js.map